@@ -19,6 +19,15 @@
 #ifndef __FILE_H
 #define __FILE_H
 
+#include <stdio.h>
+
+/*
+====================================================================
+Read all lines from file.
+====================================================================
+*/
+List* file_read_lines( FILE *file );
+
 /*
 ====================================================================
 Return a list (autodeleted strings)
@@ -28,6 +37,6 @@ Makefile stuff.
 The directoriers are marked with an asteriks.
 ====================================================================
 */
-List* dir_get_entries( char *path, char *root, char *ext );
+List* dir_get_entries( const char *path, const char *root, const char *ext );
 
 #endif
