@@ -102,8 +102,16 @@ static void show_title()
     }
     refresh_screen( 0, 0, 0, 0 );
     /* wait */
-    SDL_PumpEvents(); event_clear();
-    while ( !event_get_buttonup( &dummy, &dummy, &dummy ) ) { SDL_PumpEvents(); SDL_Delay( 20 ); }
+    SDL_PumpEvents();
+    event_clear();
+    while (1) {
+	if (event_check_quit())
+		break;
+	if (event_get_buttonup( &dummy, &dummy, &dummy ))
+		break;
+	SDL_PumpEvents();
+	SDL_Delay( 20 );
+    }
     event_clear();
     /* clear */
     free_surf(&back);
@@ -151,7 +159,7 @@ static void syntax(int argc, char *argv[])
 static void print_version()
 {
     /* display some credits */
-    printf( tr("LGeneral %s\nCopyright 2001-2011 Michael Speck\nPublished under GNU GPL\n---\n"), VERSION );
+    printf( tr("LGeneral %s\nCopyright 2001-2015 Michael Speck\nPublished under GNU GPL\n---\n"), VERSION );
     printf( tr("Looking up data in: %s\n"), get_gamedir() );
 #ifndef WITH_SOUND
     printf( tr("Compiled without sound and music\n") );
