@@ -503,7 +503,7 @@ PData* parser_read_file( const char *tree_name, const char *fname )
     ParserState *st = &state;
     PData *top = 0;
     /* open file */
-    if ( ( state.file = fopen( fname, "r" ) ) == 0 ) {
+    if ( ( state.file = fopen( fname, "rb" ) ) == 0 ) {
         sprintf( parser_error, tr("%s: file not found"), fname );
         return 0;
     }
