@@ -389,6 +389,7 @@ Unit *unit_create( Unit_Lib_Entry *prop, Unit_Lib_Entry *trsp_prop, Unit *base )
         }
     }
     /* copy the base values */
+	unit->core = base->core;
     unit->delay = base->delay;
     unit->x = base->x; unit->y = base->y;
     unit->str = base->str; unit->entr = base->entr;
@@ -1054,7 +1055,42 @@ void unit_get_df_units( Unit *unit, Unit *target, List *units, List *df_units )
         list_add( df_units, entry );
     }
 }
-
+/* get defenders for strategic bombing of players supply center */
+void unit_get_df_units_strategic( Unit *unit, Player * player,List *units, List *df_units )
+{
+    Unit *entry;
+    list_clear( df_units );
+    if ( unit->sel_prop->flags & FLYING ) {
+        list_reset( units );
+        while ( ( entry = list_next( units ) ) ) {
+            if ( entry->killed ) continue;
+            if ( entry == unit ) continue;
+            /* bombers -- intercepting impossibly covered by unit_check_attack() */
+            if ( unit_is_close( unit, entry ) )
+                if ( entry->sel_prop->flags & INTERCEPTOR )
+                    if ( player_is_ally( entry->player, player ) )
+                        if ( entry->cur_ammo > 0 ) {
+                            list_add( df_units, entry );
+                            continue;
+                        }
+            /* air-defense */
+            if ( entry->sel_prop->flags & AIR_DEFENSE )
+                if ( unit_is_close( unit, entry ) ) /* adjacent help only */
+                    if ( player_is_ally( entry->player, player ) )
+                        if ( entry->cur_ammo > 0 ) {
+                            list_add( df_units, entry );
+                            continue;
+                        }
+        }
+    }
+    /* randomly remove all support but one */
+    if (df_units->count>0)
+    {
+        entry = list_get(df_units,rand()%df_units->count);
+        list_clear(df_units);
+        list_add( df_units, entry );
+    }
+}
 /*
 ====================================================================
 Check if these two units are allowed to merge with each other.
@@ -1304,6 +1340,32 @@ Unit *unit_duplicate( Unit *unit )
     return new;
 }
 
+transferredUnitProp *unit_create_transfer_props( Unit *unit )
+{
+	transferredUnitProp *result = calloc( 1, sizeof( transferredUnitProp ) );
+
+	/* copy basic property structs
+	 * ALL POINTERS WILL BE REALLOCATED AND BROKEN IN NEXT SCENARIO! */
+	memcpy(&result->prop, &unit->prop, sizeof(unit->prop));
+	memcpy(&result->trsp_prop, &unit->trsp_prop, sizeof(unit->trsp_prop));
+	
+	snprintf( result->prop_id, sizeof(result->prop_id), "%s", unit->prop.id);
+	/* do not keep temporary air/sea transporter */
+	if( unit->trsp_prop.id && !(unit->trsp_prop.flags & SWIMMING)
+					&& !(unit->trsp_prop.flags & FLYING))
+		snprintf( result->trsp_prop_id,sizeof(result->trsp_prop_id),"%s",unit->trsp_prop.id );
+	else
+		snprintf( result->trsp_prop_id,sizeof(result->trsp_prop_id),"none" );
+	
+	snprintf( result->name, sizeof(result->name),"%s", unit->name);
+	snprintf( result->nation_id, sizeof(result->nation_id), "%s",unit->nation->id);
+	snprintf( result->player_id, sizeof(result->player_id), "%s",unit->player->id);
+	result->str = unit->str;
+	result->exp = unit->exp;
+	snprintf( result->tag, sizeof(result->tag), "%s",unit->tag);
+	return result;
+}
+
 /*
 ====================================================================
 Check if unit has low ammo or fuel.
