@@ -80,6 +80,7 @@ typedef struct _Unit {
     Unit_Lib_Entry *sel_prop;   /* selected props: either prop or trsp_prop */
     struct _Unit *backup;       /* used to backup various unit values that may temporaly change (e.g. expected losses) */
     char name[24];              /* unit name */
+	int core;					/* 1 = core, 0 = aux */
     Player *player;             /* controlling player */
     Nation *nation;             /* nation unit belongs to */
     Terrain_Type *terrain;      /* terrain the unit is currently on */
@@ -122,6 +123,26 @@ typedef struct _Unit {
                                    to the result score for attack of unit on this target */
 } __attribute((packed)) Unit;
 
+/*
+====================================================================
+Data needed to transfer unit between scenarios
+FIXME: id strings may have any length but 32 should be more than 
+enough and simplifies the handling... will break with longer ids!
+====================================================================
+*/
+typedef struct {
+	Unit_Lib_Entry prop;
+	Unit_Lib_Entry trsp_prop;
+	char prop_id[32];
+	char trsp_prop_id[32];
+	char name[24];
+	char player_id[32];
+	char nation_id[32];
+	int str;
+	int exp;
+	char tag[32];
+} transferredUnitProp;
+
 /*
 ====================================================================
 Create a unit by passing a Unit struct with the following stuff set:
@@ -286,7 +307,7 @@ that will give defensive fire to before the real battle
 ====================================================================
 */
 void unit_get_df_units( Unit *unit, Unit *target, List *units, List *df_units );
-
+void unit_get_df_units_strategic( Unit *unit, Player * player,List *units, List *df_units );
 /*
 ====================================================================
 Check if these two units are allowed to merge with each other.
@@ -368,6 +389,9 @@ Duplicate the unit.
 */
 Unit *unit_duplicate( Unit *unit );
 
+/* Duplicate properties required for transferring to next scenario. */
+transferredUnitProp *unit_create_transfer_props( Unit *unit );
+
 /*
 ====================================================================
 Check if unit has low ammo or fuel.
