@@ -20,11 +20,9 @@
 #include "event.h"
 #include "unit.h"
 #include "windows.h"
-
-enum {
-    ID_INTERN_UP = -101,
-    ID_INTERN_DOWN = -100
-};
+#include "localize.h"
+#include "purchase_dlg.h"
+#include "gui.h"
 
 /*
 ====================================================================
@@ -35,6 +33,7 @@ extern Sdl sdl;
 extern int hex_w;
 extern int old_mx, old_my;
 extern Config config;
+extern GUI *gui;
 
 /*
 ====================================================================
@@ -67,7 +66,7 @@ Write text as label contents and set hide = 0.
 If 'font' is NULL label's default font pointer is used.
 ====================================================================
 */
-void label_write( Label *label, Font *_font, char *text )
+void label_write( Label *label, Font *_font, const char *text )
 {
     Font *font = _font;
     if ( font == 0 ) font = label->def_font;
@@ -101,7 +100,7 @@ Group *group_create( SDL_Surface *frame, int alpha, SDL_Surface *img, int w, int
     group->base_id = base_id;
     group->button_limit = limit;
     if ( ( group->buttons = calloc( group->button_limit, sizeof( Button ) ) ) == 0 ) {
-        fprintf( stderr, "Out of memory\n" );
+        fprintf( stderr, tr("Out of memory\n") );
         goto failure;
     }
     group->label = label;
@@ -126,7 +125,7 @@ Add a button at x,y in group. If lock is true this button is a switch.
 'id' * group::h is the y_offset of the button.
 ====================================================================
 */
-int group_add_button( Group *group, int id, int x, int y, int lock, char *tooltip )
+int group_add_button( Group *group, int id, int x, int y, int lock, const char *tooltip )
 {
     return group_add_button_complex( group, id, id - group->base_id, 
                                      x, y, lock, tooltip );
@@ -139,10 +138,10 @@ Add a button at x,y in group. If lock is true this button is a switch.
 multiple buttons of the same icon)
 ====================================================================
 */
-int group_add_button_complex( Group *group, int id, int icon_id, int x, int y, int lock, char *tooltip )
+int group_add_button_complex( Group *group, int id, int icon_id, int x, int y, int lock, const char *tooltip )
 {
     if ( group->button_count == group->button_limit ) {
-        fprintf( stderr, "This group has reached it'S maximum number of buttons.\n" );
+        fprintf( stderr, tr("This group has reached it's maximum number of buttons.\n") );
         return 0;
     }
     group->buttons[group->button_count].surf_rect.x = x + group->frame->img->bkgnd->surf_rect.x;
@@ -155,7 +154,7 @@ int group_add_button_complex( Group *group, int id, int icon_id, int x, int y, i
     group->buttons[group->button_count].button_rect.h = group->h;
     group->buttons[group->button_count].id = id;
     group->buttons[group->button_count].active = 1;
-    strcpy_lt( group->buttons[group->button_count].tooltip, tooltip, 21 );
+    strcpy_lt( group->buttons[group->button_count].tooltip, tooltip, 31 );
     group->buttons[group->button_count].lock = lock;
     group->button_count++;
     return 1;
@@ -371,7 +370,7 @@ void edit_draw( Edit *edit )
 Set buffer and adjust cursor x.
 ====================================================================
 */
-void edit_show( Edit *edit, char *newtext )
+void edit_show( Edit *edit, const char *newtext )
 {
     int i, w = 0;
     if ( newtext )
@@ -396,7 +395,7 @@ void edit_show( Edit *edit, char *newtext )
 Modify text buffer according to unicode and keysym.
 ====================================================================
 */
-void edit_handle_key( Edit *edit, int keysym, int unicode )
+void edit_handle_key( Edit *edit, int keysym, int modifier, int unicode )
 {
     int i, changed = 0;
     switch ( keysym ) {
@@ -438,6 +437,18 @@ void edit_handle_key( Edit *edit, int keysym, int unicode )
             }
             break;
         default:
+            /* control commands */
+            if ( ( modifier & KMOD_LCTRL ) || ( modifier & KMOD_RCTRL ) ) {
+                switch ( keysym ) {
+                    case SDLK_u: /* CTRL+U: delete line */
+                        edit->text[0] = '\0';
+                        edit->cursor_pos = 0;
+                        changed = 1;
+                        break;
+                }
+                break;
+            }
+
             if ( unicode >= 32 && edit->cursor_pos < edit->limit && strlen( edit->text ) < edit->limit ) {
                 for ( i = edit->limit - 1; i > edit->cursor_pos; i-- )
                     edit->text[i] = edit->text[i - 1];
@@ -447,9 +458,10 @@ void edit_handle_key( Edit *edit, int keysym, int unicode )
         break;
     }
     if ( changed ) {
+        edit->delay = keysym != edit->last_sym ? 750 : 100;
         edit_show( edit, 0 );
-        edit->delay = 100;
         edit->last_sym = keysym;
+        edit->last_mod = modifier;
         edit->last_uni = unicode;
     }
     else
@@ -476,7 +488,7 @@ int edit_update( Edit *edit, int ms )
             edit->delay -= ms;
             if ( edit->delay <= 0 ) {
                 edit->delay = 0;
-                edit_handle_key( edit, edit->last_sym, edit->last_uni );
+                edit_handle_key( edit, edit->last_sym, edit->last_mod, edit->last_uni );
                 return 1;
             }
         }
@@ -515,8 +527,8 @@ LBox *lbox_create( SDL_Surface *frame, int alpha, int border, SDL_Surface *butto
     /* up/down */
     bx = ( frame->w / 2 - button_w ) / 2;
     by = frame->h - border - button_h;
-    group_add_button( lbox->group, ID_INTERN_UP, bx, by, 0, "Up" ); 
-    group_add_button( lbox->group, ID_INTERN_DOWN, frame->w - bx - button_w, by, 0, "Down" ); 
+    group_add_button( lbox->group, ID_INTERN_UP, bx, by, 0, tr("Up") ); 
+    group_add_button( lbox->group, ID_INTERN_DOWN, frame->w - bx - button_w, by, 0, tr("Down") ); 
     return lbox;
 failure:
     free( lbox );
@@ -545,8 +557,8 @@ void lbox_apply( LBox *lbox )
     SDL_Surface *contents = lbox->group->frame->contents;
     SDL_FillRect( contents, 0, 0x0 );
     /* items */
+    SDL_FillRect( contents, 0, 0x0 );
     if ( lbox->items ) {
-        SDL_FillRect( contents, 0, 0x0 );
         for ( i = 0; i < lbox->cell_count; i++ ) {
             item = list_get( lbox->items, i + lbox->cell_offset );
             if ( item ) {
@@ -561,8 +573,8 @@ void lbox_apply( LBox *lbox )
                 sy += lbox->cell_h + lbox->cell_gap;
             }
         }
-        frame_apply( lbox->group->frame );
     }
+    frame_apply( lbox->group->frame );
 }
 
 /*
@@ -573,13 +585,25 @@ deleted by this listbox)
 */
 void lbox_set_items( LBox *lbox, List *items )
 {
-    if ( lbox->items ) list_delete( lbox->items );
+    if ( lbox->items )
+        list_delete( lbox->items );
     lbox->items = items;
     lbox->cur_item = 0;
     lbox->cell_offset = 0;
     lbox_apply( lbox );
 }
 
+/** Select first entry in list and return pointer or NULL if empty. */
+void *lbox_select_first_item( LBox *lbox )
+{
+	if (lbox->items == NULL)
+		return NULL;
+	lbox->cur_item = list_first(lbox->items);
+	lbox->cell_offset = 0;
+	lbox_apply(lbox);
+	return lbox->cur_item;
+}
+
 /*
 ====================================================================
 handle_motion sets button if up/down has focus and returns the item
@@ -590,8 +614,10 @@ int lbox_handle_motion( LBox *lbox, int cx, int cy, void **item )
 {
     int i;
     if ( !lbox->group->frame->img->bkgnd->hide ) {
-        if ( !group_handle_motion( lbox->group, cx, cy ) && lbox->items ) {
+        if ( !group_handle_motion( lbox->group, cx, cy ) ) {
             /* check if above cell */
+            if (lbox->items == NULL)
+                return 0;
             for ( i = 0; i < lbox->cell_count; i++ )
                 if ( FOCUS( cx, cy, 
                             lbox->group->frame->img->bkgnd->surf_rect.x + lbox->cell_x,
@@ -610,13 +636,28 @@ int lbox_handle_motion( LBox *lbox, int cx, int cy, void **item )
 ====================================================================
 handle_click internally handles up/down click and returns the item
 if one was selected.
+Note: If item is clicked, @button is not changed, but @item is set.
+Thus if true is returned, @item has to be checked (@item == NULL,
+button was clicked and @button is set; @item not NULL, item was
+clicked and @button may be undefined).
 ====================================================================
 */
 int lbox_handle_button( LBox *lbox, int button_id, int cx, int cy, Button **button, void **item )
 {
     int i;
+	
     *item = 0;
+
+    /* check focus, if not in listbox nothing to do */	
+    if ( !FOCUS( cx, cy, lbox->group->frame->img->bkgnd->surf_rect.x,
+               lbox->group->frame->img->bkgnd->surf_rect.y,
+               group_get_width(lbox->group), group_get_height(lbox->group)) )
+        return 0;
+
+    /* see if up/down button was hit */	
     group_handle_button( lbox->group, button_id, cx, cy, button );
+
+    /* handle button/wheel or selection if not hidden */
     if ( !lbox->group->frame->img->bkgnd->hide ) {
     if ( ( *button && (*button)->id == ID_INTERN_UP ) || button_id == WHEEL_UP ) {
         /* scroll up */
@@ -629,7 +670,7 @@ int lbox_handle_button( LBox *lbox, int button_id, int cx, int cy, Button **butt
     else
         if ( ( *button && (*button)->id == ID_INTERN_DOWN ) || button_id == WHEEL_DOWN ) {
             /* scroll down */
-            if ( lbox->cell_count >= lbox->items->count )
+            if ( lbox->items == NULL || lbox->cell_count >= lbox->items->count )
                 lbox->cell_offset = 0;
             else {
                 lbox->cell_offset += lbox->step;
@@ -656,6 +697,16 @@ int lbox_handle_button( LBox *lbox, int button_id, int cx, int cy, Button **butt
     return 0;
 }
 
+/** Render listbox item @item (which is a simple string) to listbox cell 
+ * surface @buffer. */
+void lbox_render_text( void *item, SDL_Surface *buffer )
+{
+	const char *str = (const char*)item;
+	
+	SDL_FillRect( buffer, 0, 0x0 );
+	gui->font_std->align = ALIGN_X_LEFT | ALIGN_Y_CENTER;
+	write_text( gui->font_std, buffer, 4, buffer->h >> 1, str, 255 );
+}
 
 /*
 ====================================================================
@@ -674,7 +725,7 @@ FDlg *fdlg_create(
                    int id_ok, 
                    Label *label, 
                    void (*lbox_cb)( void*, SDL_Surface* ),
-                   void (*file_cb)( char*, SDL_Surface* ),
+                   void (*file_cb)( const char*, SDL_Surface* ),
                    SDL_Surface *surf, int x, int y )
 {
     int info_w, info_h, button_count;
@@ -698,8 +749,8 @@ FDlg *fdlg_create(
     dlg->button_y = conf_frame->h - border - conf_button_h;
     dlg->button_x = conf_frame->w;
     dlg->button_dist = 10 + conf_button_w;
-    group_add_button( dlg->group, id_ok, dlg->button_x - dlg->button_dist * 2, dlg->button_y, 0, "Ok" );
-    group_add_button( dlg->group, id_ok + 1, dlg->button_x - dlg->button_dist, dlg->button_y, 0, "Cancel" );
+    group_add_button( dlg->group, id_ok, dlg->button_x - dlg->button_dist * 2, dlg->button_y, 0, tr("Ok") );
+    group_add_button( dlg->group, id_ok + 1, dlg->button_x - dlg->button_dist, dlg->button_y, 0, tr("Cancel") );
     /* file callback */
     dlg->file_cb = file_cb;
     /* info region */
@@ -773,7 +824,7 @@ void fdlg_move( FDlg *fdlg, int x, int y )
 Add button. Graphic is taken from conf_buttons.
 ====================================================================
 */
-void fdlg_add_button( FDlg *fdlg, int id, int lock, char *tooltip )
+void fdlg_add_button( FDlg *fdlg, int id, int lock, const char *tooltip )
 {
     int x = fdlg->button_x - ( id - fdlg->group->base_id + 1 ) * fdlg->button_dist;
     group_add_button( fdlg->group, id, x, fdlg->button_y, lock, tooltip );
@@ -784,7 +835,7 @@ void fdlg_add_button( FDlg *fdlg, int id, int lock, char *tooltip )
 Show file dialogue at directory root.
 ====================================================================
 */
-void fdlg_open( FDlg *fdlg, char *root )
+void fdlg_open( FDlg *fdlg, const char *root )
 {
     strcpy( fdlg->root, root );
     fdlg->subdir[0] = 0;
@@ -886,36 +937,57 @@ SDlg *sdlg_create( SDL_Surface *list_frame, SDL_Surface *list_buttons,
                    void (*list_select_cb)(void*),
                    SDL_Surface *surf, int x, int y )
 {
-    int border = 10, alpha = 160;
+    int border = 10, alpha = 160, px, py;
     int cell_w = list_frame->w - 2 * border;
     int cell_count = ( list_frame->h - 2 * border ) / ( cell_h + 1 );
+
     SDlg *sdlg = calloc( 1, sizeof( SDlg ) );
+
+    /* listbox for players */
     if ( ( sdlg->list = lbox_create( list_frame, alpha, border, list_buttons, list_button_w, list_button_h, label,
                                      cell_count, 2, cell_w, cell_h, 1, 0x0000ff, list_render_cb, surf, x, y ) ) == 0 )
         goto failure;
+
+    /* group with human/cpu control button */
     if ( ( sdlg->ctrl = group_create( ctrl_frame, alpha, ctrl_buttons, 
                                       ctrl_button_w, ctrl_button_h, 1, id_ctrl, label, surf, x + list_frame->w - 1, y ) ) == 0 )
         goto failure;
-    group_add_button( sdlg->ctrl, id_ctrl, ctrl_frame->w - border - ctrl_button_w, ( ctrl_frame->h - ctrl_button_h ) / 2, 0, "Switch Control" );
+    group_add_button( sdlg->ctrl, id_ctrl, ctrl_frame->w - border - ctrl_button_w, ( ctrl_frame->h - ctrl_button_h ) / 2, 0, tr("Switch Control") );
+
+    /* group with ai module select button */
     if ( ( sdlg->module = group_create( mod_frame, alpha, mod_buttons, 
                                         mod_button_w, mod_button_h, 1, id_mod, label, surf, 
                                         x + list_frame->w - 1, y + ctrl_frame->h ) ) == 0 )
         goto failure;
-    group_add_button( sdlg->module, id_mod, ctrl_frame->w - border - ctrl_button_w, ( mod_frame->h - mod_button_h ) / 2, 0, "Select AI Module" );
+    group_add_button( sdlg->module, id_mod, ctrl_frame->w - border - ctrl_button_w, ( mod_frame->h - mod_button_h ) / 2, 0, tr("Select AI Module") );
 #ifndef USE_DL
     group_set_active( sdlg->module, id_mod, 0 );
 #endif
+
+    /* group with settings and confirm buttons; id_conf is id of first button
+     * in image conf_buttons */
     if ( ( sdlg->confirm = group_create( conf_frame, alpha, conf_buttons, 
-                                         conf_button_w, conf_button_h, 4, id_conf, label, surf, 
+                                         conf_button_w, conf_button_h, 6, id_conf, label, surf, 
                                          x + list_frame->w - 1, y + ctrl_frame->h + mod_frame->h ) ) == 0 )
         goto failure;
-    group_add_button( sdlg->confirm, id_conf, conf_frame->w - 1 * ( border + conf_button_w ), ( conf_frame->h - conf_button_h ) / 2, 0, "Ok" );
-    group_add_button( sdlg->confirm, id_conf + 1, border, ( conf_frame->h - conf_button_h ) / 2, 1, "Fog Of War" );
-    group_add_button( sdlg->confirm, id_conf + 2, border + ( border + conf_button_w ), ( conf_frame->h - conf_button_h ) / 2, 1, "Unit Supply" );
-    group_add_button( sdlg->confirm, id_conf + 3, border + 2 * ( border + conf_button_w ), ( conf_frame->h - conf_button_h ) / 2, 1, "Weather Influence" );
-    group_lock_button( sdlg->confirm, id_conf + 1, config.fog_of_war );
-    group_lock_button( sdlg->confirm, id_conf + 2, config.supply );
-    group_lock_button( sdlg->confirm, id_conf + 3, config.weather );
+    px = conf_frame->w - (border + conf_button_w);
+    py = (conf_frame->h - conf_button_h) / 2;
+    group_add_button( sdlg->confirm, ID_SETUP_OK, px, py, 0, tr("Ok") );
+    px = border;
+    group_add_button( sdlg->confirm, ID_SETUP_FOG, px, py, 1, tr("Fog Of War") );
+    px += border + conf_button_w;
+    group_add_button( sdlg->confirm, ID_SETUP_SUPPLY, px, py, 1, tr("Unit Supply") );
+    px += border + conf_button_w;
+    group_add_button( sdlg->confirm, ID_SETUP_WEATHER, px, py, 1, tr("Weather Influence") );
+    px += border + conf_button_w;
+    group_add_button( sdlg->confirm, ID_SETUP_DEPLOYTURN, px, py, 1, tr("Deploy Turn") );
+    px += border + conf_button_w;
+    group_add_button( sdlg->confirm, ID_SETUP_PURCHASE, px, py, 1, tr("Purchase Option") );
+    group_lock_button( sdlg->confirm, ID_SETUP_FOG, config.fog_of_war );
+    group_lock_button( sdlg->confirm, ID_SETUP_SUPPLY, config.supply );
+    group_lock_button( sdlg->confirm, ID_SETUP_WEATHER, config.weather );
+    group_lock_button( sdlg->confirm, ID_SETUP_DEPLOYTURN, config.deploy_turn );
+    group_lock_button( sdlg->confirm, ID_SETUP_PURCHASE, config.purchase );
     sdlg->select_cb = list_select_cb;
     return sdlg;
 failure:
@@ -1030,3 +1102,122 @@ int sdlg_handle_button( SDlg *sdlg, int button_id, int cx, int cy, Button **butt
     }
     return 0;
 }
+
+
+/** Select dialog:
+ * A listbox for selection with OK/Cancel buttons */
+
+SelectDlg *select_dlg_create(
+	SDL_Surface *lbox_frame, 
+	SDL_Surface *lbox_buttons, int lbox_button_w, int lbox_button_h, 
+	int lbox_cell_count, int lbox_cell_w, int lbox_cell_h,
+	void (*lbox_render_cb)(void*, SDL_Surface*),
+	SDL_Surface *conf_frame,
+	SDL_Surface *conf_buttons, int conf_button_w, int conf_button_h,
+	int id_ok, 
+	SDL_Surface *surf, int x, int y )
+{
+	SelectDlg *sdlg = NULL;
+	int sx, sy;
+	
+	sdlg = calloc(1, sizeof(SelectDlg));
+	if (sdlg == NULL)
+		goto failure;
+	
+	sdlg->button_group = group_create(conf_frame, 160, conf_buttons,
+				conf_button_w, conf_button_h, 2, id_ok, 
+				gui->label, sdl.screen, 0, 0);
+	if (sdlg->button_group == NULL)
+		goto failure;
+	sx = group_get_width( sdlg->button_group ) - 60; 
+	sy = 5;
+	group_add_button( sdlg->button_group, id_ok, sx, sy, 0, tr("Apply") );
+	group_add_button( sdlg->button_group, id_ok+1, sx + 30, sy, 0, 
+							tr("Cancel") );
+	
+	sdlg->select_lbox = lbox_create( lbox_frame, 160, 6, 
+			lbox_buttons, lbox_button_w, lbox_button_h, gui->label, 
+			lbox_cell_count, lbox_cell_count/2, 
+			lbox_cell_w, lbox_cell_h, 
+			1, 0x0000ff, lbox_render_cb, sdl.screen, 0, 0);
+	if (sdlg->select_lbox == NULL)
+		goto failure;
+	
+	return sdlg;
+failure:
+	fprintf(stderr, tr("Failed to create select dialogue\n"));
+	select_dlg_delete(&sdlg);
+	return NULL;
+}
+void select_dlg_delete( SelectDlg **sdlg )
+{
+	if (*sdlg) {
+		SelectDlg *ptr = *sdlg;
+		group_delete(&ptr->button_group);
+		lbox_delete(&ptr->select_lbox);
+		free(ptr);
+	}
+	*sdlg = NULL;
+}
+
+int select_dlg_get_width(SelectDlg *sdlg)
+{
+	return lbox_get_width(sdlg->select_lbox);
+}
+int select_dlg_get_height(SelectDlg *sdlg)
+{
+	return lbox_get_height(sdlg->select_lbox) + 
+				group_get_height(sdlg->button_group);
+}
+
+void select_dlg_move( SelectDlg *sdlg, int px, int py)
+{
+	lbox_move(sdlg->select_lbox, px, py);
+	group_move(sdlg->button_group, px, py + 
+			lbox_get_height(sdlg->select_lbox));
+}
+
+void select_dlg_hide( SelectDlg *sdlg, int value)
+{
+	group_hide(sdlg->button_group, value);
+	lbox_hide(sdlg->select_lbox, value);
+}
+void select_dlg_draw( SelectDlg *sdlg)
+{
+	group_draw(sdlg->button_group);
+	lbox_draw(sdlg->select_lbox);
+}
+void select_dlg_draw_bkgnd( SelectDlg *sdlg)
+{
+	group_draw_bkgnd(sdlg->button_group);
+	lbox_draw_bkgnd(sdlg->select_lbox);
+}
+void select_dlg_get_bkgnd( SelectDlg *sdlg)
+{
+	group_get_bkgnd(sdlg->button_group);
+	lbox_get_bkgnd(sdlg->select_lbox);
+}
+
+int select_dlg_handle_motion( SelectDlg *sdlg, int cx, int cy)
+{
+	int ret = 1;
+	void *item = NULL;
+	
+	if (!group_handle_motion(sdlg->button_group,cx,cy))
+	if (!lbox_handle_motion(sdlg->select_lbox,cx,cy,&item))
+		ret = 0;
+	return ret;
+	
+}
+
+int select_dlg_handle_button( SelectDlg *sdlg, int bid, int cx, int cy, 
+	Button **pbtn )
+{
+	void *item = NULL;
+	
+	if (group_handle_button(sdlg->button_group,bid,cx,cy,pbtn))
+		return 1;
+	if (lbox_handle_button(sdlg->select_lbox,bid,cx,cy,pbtn,&item))
+		return 0; /* internal up/down buttons */
+	return 0;
+}
