@@ -64,7 +64,7 @@ Write text as label contents and set hide = 0.
 If 'font' is NULL label's default font pointer is used.
 ====================================================================
 */
-void label_write( Label *label, Font *font, char *text );
+void label_write( Label *label, Font *font, const char *text );
 
 /*
 ====================================================================
@@ -108,7 +108,7 @@ Add a button at x,y in group. If lock is true this button is a switch.
 'id' * group::h is the y_offset of the button.
 ====================================================================
 */
-int group_add_button( Group *group, int id, int x, int y, int lock, char *tooltip );
+int group_add_button( Group *group, int id, int x, int y, int lock, const char *tooltip );
 
 /*
 ====================================================================
@@ -117,7 +117,7 @@ Add a button at x,y in group. If lock is true this button is a switch.
 multiple buttons of the same icon)
 ====================================================================
 */
-int group_add_button_complex( Group *group, int id, int icon_id, int x, int y, int lock, char *tooltip );
+int group_add_button_complex( Group *group, int id, int icon_id, int x, int y, int lock, const char *tooltip );
 
 /*
 ====================================================================
@@ -167,11 +167,14 @@ void group_draw( Group *group );
 
 /*
 ====================================================================
-Modify settings.
+Modify/Get settings.
 ====================================================================
 */
 #define group_set_surface( group, surf ) buffer_set_surface( group->frame->img->bkgnd, surf )
+#define group_get_geometry(group,x,y,w,h) buffer_get_geometry(group->frame->img->bkgnd,x,y,w,h)
 void group_move( Group *group, int x, int y );
+#define group_get_width( group ) frame_get_width( (group)->frame )
+#define group_get_height( group ) frame_get_height( (group)->frame )
 
 /*
 ====================================================================
@@ -195,6 +198,7 @@ typedef struct {
     int   cursor_pos; /* position in text */
     int   cursor_x;   /* position in label */
     int   last_sym;   /* keysym if last key (or -1 if none) */
+    int   last_mod;   /* modifier flags of last key */
     int   last_uni;   /* unicode */
     int   delay;      /* delay between key down handle key events */
 } Edit;
@@ -231,14 +235,14 @@ Set buffer and adjust cursor x. If 'newtext' is NULL the current
 text is displayed.
 ====================================================================
 */
-void edit_show( Edit *edit, char *newtext );
+void edit_show( Edit *edit, const char *newtext );
 
 /*
 ====================================================================
-Modify text buffer according to unicode and keysym.
+Modify text buffer according to unicode, keysym, and modifier.
 ====================================================================
 */
-void edit_handle_key( Edit *edit, int keysym, int unicode );
+void edit_handle_key( Edit *edit, int keysym, int modifier, int unicode );
 
 /*
 ====================================================================
@@ -296,12 +300,24 @@ Draw listbox
 
 /*
 ====================================================================
-Modify listbox settings
+Modify/Get listbox settings
 ====================================================================
 */
 #define lbox_set_surface( lbox, surf ) buffer_set_surface( lbox->group->frame->img->bkgnd, surf )
 #define lbox_move( lbox, x, y ) group_move( lbox->group, x, y )
-
+#define lbox_get_width( lbox ) group_get_width( (lbox)->group )
+#define lbox_get_height( lbox ) group_get_height( (lbox)->group )
+#define lbox_get_selected_item( lbox ) ( (lbox)->cur_item )
+#define lbox_get_selected_item_index( lbox ) \
+	(list_check( (lbox)->items, (lbox)->cur_item ))
+#define lbox_clear_selected_item( lbox ) \
+	do { (lbox)->cur_item = 0; lbox_apply(lbox); } while (0)
+void *lbox_select_first_item( LBox *lbox );
+#define lbox_is_empty( lbox ) \
+	((lbox)->items == NULL || (lbox)->items->count == 0)
+#define lbox_select_item(lbox, item) \
+	do { (lbox)->cur_item = (item); lbox_apply(lbox); } while (0)
+	
 /*
 ====================================================================
 Rebuild the listbox graphic (lbox->group->frame).
@@ -316,6 +332,7 @@ deleted by this listbox)
 ====================================================================
 */
 void lbox_set_items( LBox *lbox, List *items );
+#define lbox_clear_items( lbox ) lbox_set_items( lbox, NULL )
 
 /*
 ====================================================================
@@ -332,6 +349,11 @@ if one was selected.
 */
 int lbox_handle_button( LBox *lbox, int button_id, int cx, int cy, Button **button, void **item );
 
+/** Render listbox item @item (which is a simple string) to listbox cell 
+ * surface @buffer. */
+void lbox_render_text( void *item, SDL_Surface *buffer );
+
+
 /*
 ====================================================================
 File dialogue with space to display an info about a file
@@ -343,7 +365,7 @@ typedef struct {
     Group *group;                       /* info frame with ok/cancel buttons */
     char root[1024];                    /* root directory */
     char subdir[1024];                  /* current directory in root directory */
-    void (*file_cb)( char*, SDL_Surface* ); /* called if a file was selected
+    void (*file_cb)( const char*, SDL_Surface* ); /* called if a file was selected
                                                the full path is given */
     int info_x, info_y;                 /* info position */
     SDL_Surface *info_buffer;           /* buffer of information */
@@ -369,7 +391,7 @@ FDlg *fdlg_create(
                    int id_ok,
                    Label *label, 
                    void (*lbox_cb)( void*, SDL_Surface* ),
-                   void (*file_cb)( char*, SDL_Surface* ),
+                   void (*file_cb)( const char*, SDL_Surface* ),
                    SDL_Surface *surf, int x, int y );
 void fdlg_delete( FDlg **fdlg );
 
@@ -396,14 +418,14 @@ void fdlg_move( FDlg *fdlg, int x, int y );
 Add button. Graphic is taken from conf_buttons.
 ====================================================================
 */
-void fdlg_add_button( FDlg *fdlg, int id, int lock, char *tooltip );
+void fdlg_add_button( FDlg *fdlg, int id, int lock, const char *tooltip );
 
 /*
 ====================================================================
 Show file dialogue at directory root.
 ====================================================================
 */
-void fdlg_open( FDlg *fdlg, char *root );
+void fdlg_open( FDlg *fdlg, const char *root );
 
 /*
 ====================================================================
@@ -483,4 +505,44 @@ handle_button
 */
 int sdlg_handle_button( SDlg *sdlg, int button_id, int cx, int cy, Button **button );
 
+/*
+====================================================================
+Select dialog:
+A listbox for selection with OK/Cancel buttons 
+====================================================================
+*/
+typedef struct {
+	Group *button_group; /* okay/cancel buttons */
+	LBox *select_lbox;
+} SelectDlg;
+
+SelectDlg *select_dlg_create(
+	SDL_Surface *lbox_frame, 
+	SDL_Surface *lbox_buttons, int lbox_button_w, int lbox_button_h, 
+	int lbox_cell_count, int lbox_cell_w, int lbox_cell_h,
+	void (*lbox_render_cb)(void*, SDL_Surface*),
+	SDL_Surface *conf_frame,
+	SDL_Surface *conf_buttons, int conf_button_w, int conf_button_h,
+	int id_ok, 
+	SDL_Surface *surf, int x, int y );
+void select_dlg_delete( SelectDlg **sdlg );
+int select_dlg_get_width(SelectDlg *sdlg); 
+int select_dlg_get_height(SelectDlg *sdlg);
+void select_dlg_move( SelectDlg *sdlg, int px, int py);
+void select_dlg_hide( SelectDlg *sdlg, int value);
+void select_dlg_draw( SelectDlg *sdlg);
+void select_dlg_draw_bkgnd( SelectDlg *sdlg);
+void select_dlg_get_bkgnd( SelectDlg *sdlg);
+int select_dlg_handle_motion( SelectDlg *sdlg, int cx, int cy);
+int select_dlg_handle_button( SelectDlg *sdlg, int bid, int cx, int cy, 
+	Button **pbtn );
+#define select_dlg_set_items( sdlg, items ) \
+	lbox_set_items( sdlg->select_lbox, items )
+#define select_dlg_clear_items( sdlg ) \
+	lbox_clear_items( sdlg->select_lbox )
+#define select_dlg_get_selected_item( sdlg ) \
+	lbox_get_selected_item( sdlg->select_lbox )
+#define select_dlg_get_selected_item_index( sdlg ) \
+	lbox_get_selected_item_index( sdlg->select_lbox )
+
 #endif
